#!/usr/bin/env python
from numpy import *
from math import *
import scipy.signal as signal
import os
import getopt
import sys
import csv
from pylab import *


def usage(error=1):
    usage = """\
Version # 1.1

Usage: %s [INPUT_FILE_NAME][FILTERFACTOR_FILE_NAME]

note: input file should be in the order- Rshoulder, Rtip, Rsecondary, Lshoulder, Ltip, Lsecondary, head, tail, Rback, Lback. The file is then saved as Rshoulder, Rsecondary, Rtip, Lshoulder, Lsecondary, Ltip, head, tail, Rback, Lback.


Options:
   -h, --help    Print this message

[written by Paolo Segre, 2011]


""" % os.path.basename(sys.argv[0])

    if error:
        sys.stderr.write(usage)
        sys.exit(1)
    else:
        sys.stdout.write(usage)
        sys.exit()



def main(file,file2):

# input files

    inFile = open(file,"r")
    lines = inFile.readlines()    

    inFile2 = open(file2,"r")
    lines2 = inFile2.readlines()    


    Data= zeros(((len(lines)-1),30), float)
    Filter= zeros((1,30), float)

    for N in range(1,len(lines)):

        temp = str(lines[N]).split(',')            

        Data[N-1,:]=(float(temp[0]),float(temp[1]),float(temp[2]),float(temp[6]),float(temp[7]),float(temp[8]),float(temp[3]),float(temp[4]),float(temp[5]),float(temp[9]),float(temp[10]),float(temp[11]),float(temp[15]),float(temp[16]),float(temp[17]),float(temp[12]),float(temp[13]),float(temp[14]),float(temp[18]),float(temp[19]),float(temp[20]),float(temp[21]),float(temp[22]),float(temp[23]),float(temp[24]),float(temp[25]),float(temp[26]),float(temp[27]),float(temp[28]),float(temp[29]))
    
    

#############################

    temp2= str(lines2[1]).split(',')
    
    Filter[0,:]=(float(temp2[0]),float(temp2[1]),float(temp2[2]),float(temp2[6]),float(temp2[7]),float(temp2[8]),float(temp2[3]),float(temp2[4]),float(temp2[5]),float(temp2[9]),float(temp2[10]),float(temp2[11]),float(temp2[15]),float(temp2[16]),float(temp2[17]),float(temp2[12]),float(temp2[13]),float(temp2[14]),float(temp2[18]),float(temp2[19]),float(temp2[20]),float(temp2[21]),float(temp2[22]),float(temp2[23]),float(temp2[24]),float(temp2[25]),float(temp2[26]),float(temp2[27]),float(temp2[28]),float(temp2[29]))


#############################


    fundamental_frequency=40
    sample_rate=1000

    Output=zeros(((len(lines)-1),30), float)

    for n in range(0,30):     
        Raw=Data[:,n]
        filter_factor=Filter[0,n]
        Wn=((2*filter_factor*fundamental_frequency)/sample_rate)

        b,a = signal.butter(4,Wn,btype='low')
        Filt = signal.filtfilt(b,a,Raw)

        Output[:,n]=Filt[:]

##############################

    def plot_positions(x,y,z,name):

        figure(name)
        subplot(331)
        plot(Data[:,x])
        plot(Output[:,x],c='r')
        title(name+'_X')
    
        subplot(334)
        plot(Data[:,y])
        plot(Output[:,y],c='r')
        title(name+'_Y')

        subplot(337)
        plot(Data[:,z])
        plot(Output[:,z],c='r')
        title(name+'_Z')


        p1=subplot(332)
        plot(Data[100:150,x],Data[100:150,y])
        plot(Output[100:150,x],Output[100:150,y],c='r')
        title('100-150_XY')
        p1.set_aspect('equal', adjustable='datalim') 
    
        p2=subplot(335)
        plot(Data[100:150,x],Data[100:150,z])
        plot(Output[100:150,x],Output[100:150,z],c='r')
        title('100-150_XZ')
        p2.set_aspect('equal', adjustable='datalim') 

        p3=subplot(338)
        plot(Data[100:150,y],Data[100:150,z])
        plot(Output[100:150,y],Output[100:150,z],c='r')
        title('100-150_YZ')
        p3.set_aspect('equal', adjustable='datalim') 


        p1=subplot(333)
        plot(Data[200:250,x],Data[200:250,y])
        plot(Output[200:250,x],Output[200:250,y],c='r')
        title('200-250_XY')
        p1.set_aspect('equal', adjustable='datalim') 

        p2=subplot(336)
        plot(Data[200:250,x],Data[200:250,z])
        plot(Output[200:250,x],Output[200:250,z],c='r')
        title('200-250_XZ')
        p2.set_aspect('equal', adjustable='datalim') 

        p3=subplot(339)
        plot(Data[200:250,y],Data[200:250,z])
        plot(Output[200:250,y],Output[200:250,z],c='r')
        title('200-250_YZ')
        p3.set_aspect('equal', adjustable='datalim') 

    if 0:
        plot_positions(0,1,2,'R_Shoulder')
        plot_positions(3,4,5,'R_Primary')
        plot_positions(6,7,8,'R_Tip')
        plot_positions(9,10,11,'L_Shoulder')
        plot_positions(12,13,14,'L_Primary')
        plot_positions(15,16,17,'L_Tip')
        plot_positions(18,19,20,'Head')
        plot_positions(21,22,23,'Tail')

        show()    
######

    if 1:
        clf()
        figure(1)
        p=subplot(111)

        plot(Data[:,0],c='r')
        plot(Output[:,0],c='b') #b rshoulder
        plot(Data[:,3],c='r') 
        plot(Output[:,3],c='g') #g rprimary
        plot(Data[:,6],c='r')
        plot(Output[:,6],c='k') #k rwing
        plot(Data[:,9],c='r')
        plot(Output[:,9],c='c') #c lshoulder
        plot(Data[:,12],c='r')
        plot(Output[:,12],c='m') #m lprimary
        plot(Data[:,15],c='r')
        plot(Output[:,15],c='y') #y lwing
        plot(Data[:,18],c='r')
        plot(Output[:,18],c='orange') #y head
        plot(Data[:,21],c='r')
        plot(Output[:,21],c='brown') #y tail
        title('x')

        figure(2)
        plot(Data[:,1],c='r')
        plot(Output[:,1],c='b') #b rshoulder
        plot(Data[:,4],c='r') 
        plot(Output[:,4],c='g') #g rprimary
        plot(Data[:,7],c='r')
        plot(Output[:,7],c='k') #k rwing
        plot(Data[:,10],c='r')
        plot(Output[:,10],c='c') #c lshoulder
        plot(Data[:,13],c='r')
        plot(Output[:,13],c='m') #m lprimary
        plot(Data[:,16],c='r')
        plot(Output[:,16],c='y') #y lwing
        plot(Data[:,19],c='r')
        plot(Output[:,19],c='orange') #y head
        plot(Data[:,22],c='r')
        plot(Output[:,22],c='brown') #y tail
        title('y')


        figure(3)
        plot(Data[:,2],c='r')
        plot(Output[:,2],c='b') #b rshoulder
        plot(Data[:,5],c='r') 
        plot(Output[:,5],c='g') #g rprimary
        plot(Data[:,8],c='r')
        plot(Output[:,8],c='k') #k rwing
        plot(Data[:,11],c='r')
        plot(Output[:,11],c='c') #c lshoulder
        plot(Data[:,14],c='r')
        plot(Output[:,14],c='m') #m lprimary
        plot(Data[:,17],c='r')
        plot(Output[:,17],c='y') #y lwing
        plot(Data[:,20],c='r')
        plot(Output[:,20],c='orange') #y head
        plot(Data[:,23],c='r')
        plot(Output[:,23],c='brown') #y tail
        title('z')
    
        show()


#    Write Coordinates
    Writer = csv.writer(open(file[:-10]+"_filt.csv", 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
    Writer.writerow(['RShoulder_X','RShoulder_Y','RShoulder_Z','RPrimary_X','RPrimary_Y','RPrimary_Z','RWing_X','RWing_Y','RWing_Z','LShoulder_X','LShoulder_Y','LShoulder_Z','LPrimary_X','LPrimary_Y','LPrimary_Z','LWing_X', 'LWing_Y','LWing_Z','Head_X','Head_Y','Head_Z','Tail_X','Tail_Y','Tail_Z','RBack_X','RBack_Y','RBack_Z','LBack_X','LBack_Y','LBack_Z'])
    for N in range(0,len(Data[:,0])):
        Writer.writerow(Output[N])


######################################################################################################################################################
    


if __name__ == '__main__':

    try:
        opts, args = getopt.getopt(sys.argv[1:], 'h', ['help'])
    

    except getopt.error:
        sys.stderr.write("%s: %s \nTry `%s --help` for more informatIon\n"
            % (PROGNAME, e, PROGNAME))
        sys.exit(1)


    for o in opts:
        if o in ('-h', '--help'):
            usage(error=0)

        if len(args) != 2:
            usage()

    main(args[0],args[1])
