#!/usr/bin/env python
from numpy import *
from math import *
from scipy import interpolate
import scipy
import os
import getopt
import sys
import csv
from pylab import *


def usage(error=1):
    usage = """\
Version # 7.0 5-28-2014

Usage: %s [INPUT_FILE_NAME][INPUT_FILE_NAME2]

Input values should be saved in the second line of a comma_separated_value (.csv) file as follows:

RShoulder_X, RShoulder_Y, RShoulder_Z, RPrimary_X, RPrimary_Y, RPrimary_Z, RWing_X, RWing_Y, RWing_Z, LShoulder_X, LShoulder_Y, LShoulder_Z, LPrimary_X, LPrimary_Y, LPrimary_Z, LWing_X, LWing_Y, LWing_Z, Head_X, Head_Y, Head_Z, Tail_X, Tail_Y, Tail_Z, RBack_X, RBack_Y, RBackZ, LBack_X, LBack_Y, LBack_Z

Second file should contain 2 rows, with the data starting in the second row. This should be the frame where Pronation and Supination start. It MUST start with a Pronation and end with a Pronation

---note: Change FrameMult (line 847) to convert wingtip velocity from mm/frame to whatever you want (e.g.: 10 = [1000fps*10 upsampled frames]/[1000mm/m])
    
----------------
Changelog, new in v7:
-added angle of attack velocity
-added angular velocity of wingtip
-calculates instantaneous angles and saves to file
-cleaned up many of the problems listed below
-cleaned up lots of unneeded files
-added roll
-cleaned up very messy way of returning strokeplane variables 
----------------
Changelog, new in v6:

-added primary feathers
-added angle of attack calculations
-added averaged angle of attack, in average graphs
-added AngleAttack.csv in main folder
-changed the way averaged files are caculated.
-removed left minus right files
-removed angle figures
 
for now this program cannot deal with angle of attacks greater than 180 degrees, but it can deal with angles less than 0. There is no data where the angle of attack during upstroke is greater than 180. (Lines 1084 and 1108)

Options:
   -h, --help    Print this message

[written by Paolo Segre, 2010]


""" % os.path.basename(sys.argv[0])

    if error:
        sys.stderr.write(usage)
        sys.exit(1)
    else:
        sys.stdout.write(usage)
        sys.exit()



def main(file,time):
    seterr(all='ignore')
    (path, filename) = os.path.split(file)

# input files

    inFile = open(file,"r")
    lines = inFile.readlines()    

    inFile2 = open(time,"r")
    lines2 = inFile2.readlines()    


    Data= zeros(((len(lines)-1),30), float)



    for N in range(1,len(lines)):

        temp = str(lines[N]).split(',')            

        Data[N-1,:]=(float(temp[0]),float(temp[1]),float(temp[2]),float(temp[3]),float(temp[4]),float(temp[5]),float(temp[6]),float(temp[7]),float(temp[8]),float(temp[9]),float(temp[10]),float(temp[11]),float(temp[12]),float(temp[13]),float(temp[14]),float(temp[15]),float(temp[16]),float(temp[17]),float(temp[18]),float(temp[19]),float(temp[20]),float(temp[21]),float(temp[22]),float(temp[23]),float(temp[24]),float(temp[25]),float(temp[26]),float(temp[27]),float(temp[28]),float(temp[29]))
    


    temp2= str(lines2[1]).split(',')
    U=[]
    
    for N in range(0,len(temp2)):    
        U.append(int(temp2[N]))        
        

# Make directory for figures
    os.system("mkdir %s"%(path+"/wingbeat_figures"))
    os.system("mkdir %s"%(path+"/wingbeat_timecourse"))
    os.system("mkdir %s"%(path+"/average_graphs"))
# Define Output Arrays

    Output= zeros(((U[len(U)-1]-1),17))

# Read data from input array


    RSHOULDER= zeros(((len(lines)-1),3))
    RPRIMARY= zeros(((len(lines)-1),3))
    RWING= zeros(((len(lines)-1),3))
    LSHOULDER= zeros(((len(lines)-1),3))
    LPRIMARY= zeros(((len(lines)-1),3))
    LWING= zeros(((len(lines)-1),3))
    HEAD= zeros(((len(lines)-1),3))
    TAIL= zeros(((len(lines)-1),3))
    RBACK= zeros(((len(lines)-1),3))
    LBACK= zeros(((len(lines)-1),3))
    

    RSHOULDER[:,0:3]=Data[:,0:3]
    RPRIMARY[:,0:3]=Data[:,3:6]
    RWING[:,0:3]=Data[:,6:9]
    LSHOULDER[:,0:3]=Data[:,9:12]
    LPRIMARY[:,0:3]=Data[:,12:15]
    LWING[:,0:3]=Data[:,15:18]
    HEAD[:,0:3]=Data[:,18:21]
    TAIL[:,0:3]=Data[:,21:24]    
    RBACK[:,0:3]=Data[:,24:27]
    LBACK[:,0:3]=Data[:,27:30]


# Center head at 0,0,- = subtract head (XY) coordinates from all vectors 

    Head1=HEAD-HEAD
    Head1[:,2]=HEAD[:,2]
    
    RShoulder1=RSHOULDER-HEAD    
    RShoulder1[:,2]=RSHOULDER[:,2]
    RPrimary1=RPRIMARY-HEAD
    RPrimary1[:,2]=RPRIMARY[:,2]    
    RWing1=RWING-HEAD
    RWing1[:,2]=RWING[:,2]


    
    LShoulder1=LSHOULDER-HEAD
    LShoulder1[:,2]=LSHOULDER[:,2]
    LPrimary1=LPRIMARY-HEAD
    LPrimary1[:,2]=LPRIMARY[:,2]
    LWing1=LWING-HEAD
    LWing1[:,2]=LWING[:,2]
    Tail1=TAIL-HEAD
    Tail1[:,2]=TAIL[:,2]

    RBack1=RBACK-HEAD
    RBack1[:,2]=RBACK[:,2]

    LBack1=LBACK-HEAD
    LBack1[:,2]=LBACK[:,2]



# Define global output data matrices
    RShoulder= zeros(((U[len(U)-1]-1),3))
    RPrimary= zeros(((U[len(U)-1]-1),3))
    RWing= zeros(((U[len(U)-1]-1),3))
    LShoulder= zeros(((U[len(U)-1]-1),3))
    LPrimary= zeros(((U[len(U)-1]-1),3))
    LWing= zeros(((U[len(U)-1]-1),3))
    Head= zeros(((U[len(U)-1]-1),3))
    Tail= zeros(((U[len(U)-1]-1),3))
    RBack= zeros(((U[len(U)-1]-1),3))
    LBack= zeros(((U[len(U)-1]-1),3))



# Transform all coordinates to Wingtip frame


    Rotation_Angle = ['Rotation_Angle']
    Rotation_Angle_DS = ['Rotation_Angle_DS']
    Rotation_Angle_US =['Rotation_Angle_US']    
    Time_Wingbeat = ['Time_Wingbeat']
    Time_DS =['Time_DS']
    Time_US    =['Time_US']    


   # Calculate Rotation angle for every downstroke and upstroke

    for n in range((len(U)-1)):
        
        
        A = ((U[n]-1))    
        B = (U[n+1]-1)    
        Midpoint_Start = (RWing1[A]+LWing1[A])/2
        Midpoint_Start[2]=0

        Midpoint_End = (RWing1[B]+LWing1[B])/2
        Midpoint_End[2] = 0

        if n%2==0:
            Mid = (Midpoint_End-Midpoint_Start)
        else:
            Mid = (Midpoint_Start-Midpoint_End)
        
        LengthMid= sqrt(Mid[0]**2 + Mid[1]**2)

        Xaxis=(1,0,0)
        
    
        
   # Calculate rotation angle between Midpoint and Xaxis vector. Create adjustment factor for acos: if Y is positive, subtract angle from 360 degrees to find rotation angle        
        
        if Mid[1]<0:
            ZRotationAngle= radians(360)-acos(((dot(Mid,Xaxis))/(LengthMid)))
        
        else: 
            ZRotationAngle= acos(((dot(Mid,Xaxis))/(LengthMid)))


        Rotation_Angle.append(360-(degrees(ZRotationAngle)))
        Time_Wingbeat.append(B-A)
    

#    Create rotation matrix, and multiply times Body1 and RShoulder1 vectors

        RotZ = array(((cos(ZRotationAngle),sin(ZRotationAngle),0),(-sin(ZRotationAngle),cos(ZRotationAngle),0),(0,0,1)))
        
        RShoulder2=dot(RotZ,transpose(RShoulder1[A:B,:]))
        RPrimary2=dot(RotZ,transpose(RPrimary1[A:B,:]))
        RWing2=dot(RotZ,transpose(RWing1[A:B,:]))
        LShoulder2=dot(RotZ,transpose(LShoulder1[A:B,:]))
        LPrimary2=dot(RotZ,transpose(LPrimary1[A:B,:]))
        LWing2=dot(RotZ,transpose(LWing1[A:B,:]))
        Head2=dot(RotZ,transpose(Head1[A:B,:]))
        Tail2=dot(RotZ,transpose(Tail1[A:B,:]))
        RBack2=dot(RotZ,transpose(RBack1[A:B,:]))
        LBack2=dot(RotZ,transpose(LBack1[A:B,:]))

        RShoulder3=transpose(RShoulder2)
        RPrimary3=transpose(RPrimary2)
        RWing3=transpose(RWing2)
        LWing3=transpose(LWing2)
        LPrimary3=transpose(LPrimary2)
        LShoulder3=transpose(LShoulder2)        
        Head3=transpose(Head2)
        Tail3=transpose(Tail2)
        RBack3=transpose(RBack2)
        LBack3=transpose(LBack2)



        RShoulder[A:B,:]=RShoulder3[:,:]
        RPrimary[A:B,:]=RPrimary3[:,:]
        RWing[A:B,:]=RWing3[:,:]
        LShoulder[A:B,:]=LShoulder3[:,:]
        LPrimary[A:B,:]=LPrimary3[:,:]
        LWing[A:B,:]= LWing3[:,:]
        Head[A:B,:]= Head3[:,:]
        Tail[A:B,:]=Tail3[:,:]
        RBack[A:B,:]=RBack3[:,:]           
        LBack[A:B,:]=LBack3[:,:]



# Create output array for transformed coordinates
        
        Transformed=hstack((RShoulder,RPrimary, RWing, LShoulder,LPrimary, LWing, Head, Tail,RBack,LBack))
    
        
#TRANSFORMATION COMPLETED

##############################################################################################################################


    Avg_Rot = average(Rotation_Angle[1:])    
    x = [-100,0,100]
    y = [-100*tan(radians(Avg_Rot)),0,100*tan(radians(Avg_Rot))]


    for n in range(1,(len(Rotation_Angle))):
        if n%2==1:

            Rotation_Angle_DS.append(90-Rotation_Angle[n])
        
        else:    

            Rotation_Angle_US.append(90-Rotation_Angle[n])        
                

    for n in range(1,(len(Time_Wingbeat))):
        if n%2==1:
            Time_DS.append(Time_Wingbeat[n])
        
        else:
            Time_US.append(Time_Wingbeat[n])
    
###################################################################################################################################33        
# Body Angle: Shoulder midpoint to rump

    BodyXYZ=((RShoulder+LShoulder)/2)-Tail

    BodyXZ= zeros(((len(BodyXYZ)),3), float)
    BodyYZ= zeros(((len(BodyXYZ)),3), float)        
    
    BodyXZ[:,0]=BodyXYZ[:,0]
    BodyXZ[:,1]=0
    BodyXZ[:,2]=BodyXYZ[:,2]
    

    LengthBodyXZ=sqrt((BodyXYZ[:,0]**2)+(BodyXYZ[:,2]**2))

    Zaxis=array((0,0,1))
    



    BodyAngleXZ= zeros(((len(BodyXYZ)),1), float)    


    for n in range(len(BodyXYZ)):

        if BodyXYZ[n,0]>0:
            BodyAngleXZ[n]= acos((dot(BodyXZ[n,:],Zaxis))/(LengthBodyXZ[n]))
    
        else:
            BodyAngleXZ[n]= -acos((dot(BodyXZ[n,:],Zaxis))/(LengthBodyXZ[n]))
    


    BodyYZ[:,0]=0
    BodyYZ[:,1]=BodyXYZ[:,1]
    BodyYZ[:,2]=BodyXYZ[:,2]

    LengthBodyYZ=sqrt((BodyXYZ[:,1]**2)+(BodyXYZ[:,2]**2))

    BodyAngleYZ= zeros(((len(BodyXYZ)),1), float)    


    for n in range(len(BodyXYZ)):

        if BodyXYZ[n,1]>0:
            BodyAngleYZ[n]= acos((dot(BodyYZ[n,:],Zaxis))/(LengthBodyYZ[n]))
    
        else:
            BodyAngleYZ[n]= -acos((dot(BodyYZ[n,:],Zaxis))/(LengthBodyYZ[n]))

    Output[:,0]=(90-degrees(BodyAngleXZ[:,0]))
    Output[:,1]=(degrees(BodyAngleYZ[:,0]))

##########################################################################################################
# calculate roll
    #roll is calculated by making 2 transformations to align the body axis of the transformed data to the x axis. The vector that is transformed to the body frame of reference is the roll vector- which is the cross product of Rback, LBack, Tail. Then roll is calculated as angle between twice transformed roll_vector and the Zaxis  

    Roll_vector = cross(LBack-RBack,Tail-LBack)

    Roll_vector2=empty_like(Roll_vector)
    Roll_vector3=empty_like(Roll_vector)
    Roll_Angle=empty_like(Roll_vector[:,0])


    for n in range(0,len(BodyAngleXZ)):
        RotYmat = array(((cos((-pi/2)+BodyAngleXZ[n]),0,-sin((-pi/2)+BodyAngleXZ[n])),(0,1,0),(sin((-pi/2)+BodyAngleXZ[n]),0,cos((-pi/2)+BodyAngleXZ[n]))))
        Roll_vector2[n]=dot(RotYmat,transpose(Roll_vector[n,:]))

    for n in range(0,len(BodyAngleYZ)):
        RotZmat = array(((cos(BodyAngleYZ[n]),sin(BodyAngleYZ[n]),0),(-sin(BodyAngleYZ[n]),cos(BodyAngleYZ[n]),0),(0,0,1)))
        Roll_vector3[n]=dot(RotZmat,transpose(Roll_vector2[n,:]))

    Roll_vector3[:,0]=0

    for n in range(len(Roll_vector3)):
        LengthRollVector=sqrt((Roll_vector3[n,1]**2)+(Roll_vector3[n,2]**2))
        if Roll_vector3[n,1]>0:
            Roll_Angle[n]= degrees(acos((dot(Roll_vector3[n,:],[0,0,1]))/(LengthRollVector)))
    
        else:
            Roll_Angle[n]= degrees(-acos((dot(Roll_vector3[n,:],[0,0,1]))/(LengthRollVector)))

##########################################################################################################
#plot global XY
    if 1:
        clf()
        plot(LWING[:,0],LWING[:,1],c='r')
        plot(LSHOULDER[:,0],LSHOULDER[:,1],c='y')
        plot(LPRIMARY[:,0],LPRIMARY[:,1],c='m')
        plot(RWING[:,0],RWING[:,1],c='b')
        plot(RSHOULDER[:,0],RSHOULDER[:,1],c='g')
        plot(RPRIMARY[:,0],RPRIMARY[:,1],c='c')
        plot(HEAD[:,0],HEAD[:,1],c='k')
        plot(TAIL[:,0],TAIL[:,1],c='Orange')
        plot(RBACK[:,0],RBACK[:,1],c='c')
        plot(LBACK[:,0],LBACK[:,1],c='m')
        axes().set_aspect('equal', 'datalim')
        grid(True)
#       show()    
        savefig(path+'/true_global_XY_'+filename[:-4]+'.png')

###########################################################
# plot all points 

        clf()
        plot(LPrimary[U[0]:U[-1],0],LPrimary[U[0]:U[-1],2],c='m')
        plot(LWing[U[0]:U[-1],0],LWing[U[0]:U[-1],2],c='r')
        plot(LShoulder[U[0]:U[-1],0],LShoulder[U[0]:U[-1],2],c='y')
        plot(RPrimary[U[0]:U[-1],0],RPrimary[U[0]:U[-1],2],c='c')
        plot(RWing[U[0]:U[-1],0],RWing[U[0]:U[-1],2],c='b')
        plot(RShoulder[U[0]:U[-1],0],RShoulder[U[0]:U[-1],2],c='g')
        plot(Head[U[0]:U[-1],0],Head[U[0]:U[-1],2],c='k')
        plot(Tail[U[0]:U[-1],0],Tail[U[0]:U[-1],2],c='Orange')
        plot(RBack[U[0]:U[-1],0],RBack[U[0]:U[-1],2],c='c')
        plot(LBack[U[0]:U[-1],0],LBack[U[0]:U[-1],2],c='m')   
        axis((-85,65,-100,50))    
        axes().set_aspect('equal', 'datalim')
        grid(True)
#       show()    
        savefig(path+'/XZ_global_'+filename[:-4]+'.png')

        clf()
        plot(LPrimary[U[0]:U[-1],1],LPrimary[U[0]:U[-1],2],c='m')
        plot(LWing[U[0]:U[-1],1],LWing[U[0]:U[-1],2],c='r')
        plot(LShoulder[U[0]:U[-1],1],LShoulder[U[0]:U[-1],2],c='y')
        plot(RPrimary[U[0]:U[-1],1],RPrimary[U[0]:U[-1],2],c='c')
        plot(RWing[U[0]:U[-1],1],RWing[U[0]:U[-1],2],c='b')
        plot(RShoulder[U[0]:U[-1],1],RShoulder[U[0]:U[-1],2],c='g')
        plot(Head[U[0]:U[-1],1],Head[U[0]:U[-1],2],c='k')
        plot(Tail[U[0]:U[-1],1],Tail[U[0]:U[-1],2],c='Orange')
        plot(RBack[U[0]:U[-1],1],RBack[U[0]:U[-1],2],c='c')
        plot(LBack[U[0]:U[-1],1],LBack[U[0]:U[-1],2],c='m')
        axis((-75,75,-100,50))    
        axes().set_aspect('equal', 'datalim')
        grid(True)
#       show()    
        savefig(path+'/YZ_global_'+filename[:-4]+'.png')

        clf()
        plot(LPrimary[U[0]:U[-1],0],LPrimary[U[0]:U[-1],1],c='m')
        plot(LWing[U[0]:U[-1],0],LWing[U[0]:U[-1],1],c='r')
        plot(LShoulder[U[0]:U[-1],0],LShoulder[U[0]:U[-1],1],c='y')
        plot(RPrimary[U[0]:U[-1],0],RPrimary[U[0]:U[-1],1],c='c')
        plot(RWing[U[0]:U[-1],0],RWing[U[0]:U[-1],1],c='b')
        plot(RShoulder[U[0]:U[-1],0],RShoulder[U[0]:U[-1],1],c='g')
        plot(Head[U[0]:U[-1],0],Head[U[0]:U[-1],1],c='k')
        plot(Tail[U[0]:U[-1],0],Tail[U[0]:U[-1],1],c='Orange')
        plot(RBack[U[0]:U[-1],0],RBack[U[0]:U[-1],1],c='c')
        plot(LBack[U[0]:U[-1],0],LBack[U[0]:U[-1],1],c='m')
        plot(x,y)
    
        axis((-85,65,-75,75))
        axes().set_aspect('equal', 'datalim')
        grid(True)
#       show()    
        savefig(path+'/XY_global_'+filename[:-4]+'.png')

#       Write Transformed Coordinates
        Writer = csv.writer(open(path+"/trans_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
        Writer.writerow(['RShoulder_X','RShoulder_Y','RShoulder_Z','RPrimary_X','RPrimary_Y','RPrimary_Z','RWing_X','RWing_Y','RWing_Z','LShoulder_X','LShoulder_Y','LShoulder_Z','LPrimary_X','LPrimary_Y','LPrimary_Z','LWing_X', 'LWing_Y','LWing_Z','Head_X','Head_Y','Head_Z','Tail_X','Tail_Y','Tail_Z','Rback_X','Rback_Y','Rback_Z','LBack_X','LBack_Y','LBack_Z'])
        for N in range(0,U[len(U)-1]-1):
            Writer.writerow(Transformed[N])

#######################################################################################################
    #average wingangle
    #####
    def angleofattack(input):
 
        number=len(input[:,0])
        
        RAngleAttack=[]
        RLeadingEdge=input[:,6:9]-input[:,0:3]
        RTrailingEdge=input[:,3:6]-input[:,6:9]


        RLeadingEdgePerp=vstack((-RLeadingEdge[:,1],RLeadingEdge[:,0],zeros_like(RLeadingEdge[:,2]))).T
        RWing_direction_motion=cross(RLeadingEdge,RLeadingEdgePerp)

        RWing_plane = cross(RTrailingEdge,RLeadingEdge)

        for n in range(0,number):
            RWing_plane[n,:]=RWing_plane[n,:]/norm(RWing_plane[n,:])

            RAngleAttack_tmp=degrees(acos(dot(RWing_direction_motion[n,:],RWing_plane[n,:])/(norm(RWing_direction_motion[n,:])*norm(RWing_plane[n,:]))))

            if RWing_plane[n,2]>0:                           
                if dot(RWing_plane[n,:],RLeadingEdgePerp[n,:])>0:
                    RAngleAttack_tmp=-RAngleAttack_tmp

            RAngleAttack=append(RAngleAttack,RAngleAttack_tmp)


        LAngleAttack=[]
        LLeadingEdge=input[:,15:18]-input[:,9:12]
        LTrailingEdge=input[:,12:15]-input[:,15:18]

        LLeadingEdgePerp=vstack((-LLeadingEdge[:,1],LLeadingEdge[:,0],zeros_like(LLeadingEdge[:,2]))).T
        LWing_direction_motion=cross(LLeadingEdge,LLeadingEdgePerp)


        LWing_plane = cross(LLeadingEdge,LTrailingEdge)

        for n in range(0,number):
            LWing_plane[n,:]=LWing_plane[n,:]/norm(LWing_plane[n,:])

            LAngleAttack_tmp=degrees(acos(dot(LWing_direction_motion[n,:],LWing_plane[n,:])/(norm(LWing_direction_motion[n,:])*norm(LWing_plane[n,:]))))

            if LWing_plane[n,2]>0:                           
                if dot(LWing_plane[n,:],LLeadingEdgePerp[n,:])<0:
                    LAngleAttack_tmp=-LAngleAttack_tmp

            LAngleAttack=append(LAngleAttack,LAngleAttack_tmp)

        return RAngleAttack,LAngleAttack

# Calculate Angles relative to all strokeplanes

    def StrokeAngle(R,S,T):


    # Calculate Reduced Major Axis Regression using RWing columns X and Z

        if T==1:

            correlation_coefficient = corrcoef([RWing[R:S,0],RWing[R:S,2]])

            if correlation_coefficient[0,1]<0:
                sign=-1
            else:
                sign=1


            Mean1= mean(RWing[R:S,0])

            Std1= std(RWing[R:S,0])

            Mean2=mean(RWing[R:S,2])
            Std2=std(RWing[R:S,2])

            A= sign*Std2/Std1
            B= Mean2-(A*Mean1)

            x=array((-100,0,100))    
            y=(A*x)+B
            y2=(A*x)
    


        #angle to horizontal

            Xaxis=array((1,0,0))

            StrokeVector=(x[2],0,y2[2])
            LengthStrokeVector=sqrt((StrokeVector[0]**2)+(StrokeVector[2]**2))

            if StrokeVector[2]>0:
                StrokePlaneAngle= -acos(((dot(StrokeVector,Xaxis))/(LengthStrokeVector)))

            else:
                StrokePlaneAngle= acos(((dot(StrokeVector,Xaxis))/(LengthStrokeVector)))
    
            RotStrokeAngle = ([(cos(StrokePlaneAngle), 0, -sin(StrokePlaneAngle)),(0,1,0),(sin(StrokePlaneAngle),0,cos(StrokePlaneAngle))])
    

    # Calculate Reduced Major Axis Regression using LWing columns X and Z

        elif T==2:

            correlation_coefficient = corrcoef([LWing[R:S,0],LWing[R:S,2]])

            if correlation_coefficient[0,1]<0:
                sign=-1
            else:
                sign=1

            Mean1= mean(LWing[R:S,0])

            Std1= std(LWing[R:S,0])

            Mean2=mean(LWing[R:S,2])
            Std2=std(LWing[R:S,2])

            A= sign*Std2/Std1
            B= Mean2-(A*Mean1)

            x=array((-100,0,100))    
            y=(A*x)+B
            y2=(A*x)

        #angle to horizontal

            Xaxis=array((1,0,0))

            StrokeVector=(x[2],0,y2[2])
            LengthStrokeVector=sqrt((StrokeVector[0]**2)+(StrokeVector[2]**2))

            if StrokeVector[2]>0:
                StrokePlaneAngle= -acos(((dot(StrokeVector,Xaxis))/(LengthStrokeVector)))

            else:
                StrokePlaneAngle= acos(((dot(StrokeVector,Xaxis))/(LengthStrokeVector)))


            RotStrokeAngle = ([(cos(StrokePlaneAngle), 0, -sin(StrokePlaneAngle)),(0,1,0),(sin(StrokePlaneAngle),0,cos(StrokePlaneAngle))])

       
        # Rotate around Y axis,


        RShoulder2=transpose(dot(RotStrokeAngle,transpose(RShoulder)))
        RWing2=transpose(dot(RotStrokeAngle,transpose(RWing)))
        LShoulder2=transpose(dot(RotStrokeAngle,transpose(LShoulder)))
        LWing2=transpose(dot(RotStrokeAngle,transpose(LWing)))
        Head2=transpose(dot(RotStrokeAngle,transpose(Head)))
        Tail2=transpose(dot(RotStrokeAngle,transpose(Tail)))

    # calculate deviation from stroke plane, distance of shoulder to stroke plane
    
        if T==1:    
            Deviation=RWing2[:,2]-B        
            ShoulderDist=B-RShoulder2[:,2]


        else:
            Deviation=LWing2[:,2]-B
            ShoulderDist=B-LShoulder2[:,2]        

    #position and angle of wing to stroke plane 

        #Project point into XY plane (remember stroke plane has been rotated to be parallel to XY plane)

        if T==1:
            RWing2XYZ=RWing2-RShoulder2
            LengthRWing2XYZ=sqrt((RWing2XYZ[:,0]**2)+(RWing2XYZ[:,1]**2)+(RWing2XYZ[:,2]**2))    
    
            RWing2XY=RWing2-RShoulder2
            RWing2XY[:,2]=0
            LengthRWing2XY=sqrt((RWing2XY[:,0]**2)+(RWing2XY[:,1]**2)+(RWing2XY[:,2]**2))

            WingAngle=arccos((RWing2XYZ[:,0]*RWing2XY[:,0]+RWing2XYZ[:,1]*RWing2XY[:,1])/((LengthRWing2XYZ)*(LengthRWing2XY)))*np.sign([RWing2XYZ[:,2]])


        else:
            LWing2XYZ=LWing2-LShoulder2
            LengthLWing2XYZ=sqrt((LWing2XYZ[:,0]**2)+(LWing2XYZ[:,1]**2)+(LWing2XYZ[:,2]**2))    
    
            LWing2XY=LWing2-LShoulder2
            LWing2XY[:,2]=0
            LengthLWing2XY=sqrt((LWing2XY[:,0]**2)+(LWing2XY[:,1]**2)+(LWing2XY[:,2]**2))

            WingAngle=arccos((LWing2XYZ[:,0]*LWing2XY[:,0]+LWing2XYZ[:,1]*LWing2XY[:,1])/((LengthLWing2XYZ)*(LengthLWing2XY)))*np.sign([LWing2XYZ[:,2]])



    # calculate position relative to strokeplane
        NegXaxis=(-1,0,0)

        if T==1:
            Position=arccos((RWing2XY[:,0]*NegXaxis[0]+RWing2XY[:,1]*NegXaxis[1]+RWing2XY[:,2]*NegXaxis[2])/(LengthRWing2XY))
        else:            
            Position=arccos((LWing2XY[:,0]*NegXaxis[0]+LWing2XY[:,1]*NegXaxis[1]+LWing2XY[:,2]*NegXaxis[2])/(LengthLWing2XY))


    # position and angle of wing relative to horizontal line through shoulder
        

        #Transform Shoulder and Wingtip to Shoulder centered Frame Of Reference
        
        RShoulder_Sh = RShoulder-RShoulder
        RWing_Sh = RWing - RShoulder

        LShoulder_Sh = LShoulder-LShoulder
        LWing_Sh = LWing - LShoulder


        #Project Wingtip onto XY Plane

        RWing_ShXY = RWing - RShoulder
        RWing_ShXY[:,2]=0    

        LWing_ShXY = LWing - LShoulder
        LWing_ShXY[:,2]=0

        if T==1:
            Length_RWing_Sh = sqrt((RWing_Sh[:,0]**2)+(RWing_Sh[:,1]**2)+(RWing_Sh[:,2]**2))
            Length_RWing_ShXY = sqrt((RWing_ShXY[:,0]**2)+(RWing_ShXY[:,1]**2)+(RWing_ShXY[:,2]**2))


            WingAngle_Ground= arccos((RWing_Sh[:,0]*RWing_ShXY[:,0]+RWing_Sh[:,1]*RWing_ShXY[:,1])/((Length_RWing_Sh)*(Length_RWing_ShXY)))*np.sign([RWing_Sh[:,2]])

            Position_Ground= arccos((RWing_ShXY[:,0]*NegXaxis[0]+RWing_ShXY[:,1]*NegXaxis[1]+RWing_ShXY[:,2]*NegXaxis[2])/(Length_RWing_ShXY))


        else:
            Length_LWing_Sh = sqrt((LWing_Sh[:,0]**2)+(LWing_Sh[:,1]**2)+(LWing_Sh[:,2]**2))
            Length_LWing_ShXY = sqrt((LWing_ShXY[:,0]**2)+(LWing_ShXY[:,1]**2)+(LWing_ShXY[:,2]**2))


            WingAngle_Ground= arccos((LWing_Sh[:,0]*LWing_ShXY[:,0]+LWing_Sh[:,1]*LWing_ShXY[:,1])/((Length_LWing_Sh)*(Length_LWing_ShXY)))*np.sign([LWing_Sh[:,2]])

            Position_Ground= arccos((LWing_ShXY[:,0]*NegXaxis[0]+LWing_ShXY[:,1]*NegXaxis[1]+LWing_ShXY[:,2]*NegXaxis[2])/(Length_LWing_ShXY))
        

        WingAngle_Ground = nan_to_num(WingAngle_Ground)


# Plot per wingbeat XZ graphs 
        
        figure(1)
        clf()
        plot(x,y, c='g')
            
        if T==2:
            plot(LWing[R:S,0],LWing[R:S,2],c='r')
            plot(LShoulder[R:S,0],LShoulder[R:S,2],c='y')
            

        elif T==1:
            plot(RWing[R:S,0],RWing[R:S,2],c='b')
            plot(RShoulder[R:S,0],RShoulder[R:S,2],c='g')


        axis((-85,65,-100,50))        
        axes().set_aspect('equal', 'datalim')
        grid(True)
#        show()
        savefig(path+"/wingbeat_figures/"+Q+".svg")



# Wingtip distance traveled

        Wingtip_Dist_Trav=0

        for n in range(R,S-1):
            if T==1:
                
                Wingtip_Dist_Trav=Wingtip_Dist_Trav+(sqrt(((RWing[n+1,0]-RWing[n,0])**2)+((RWing[n+1,1]-RWing[n,1])**2)+((RWing[n+1,2]-RWing[n,2])**2)))
        
            
            else:
                Wingtip_Dist_Trav=Wingtip_Dist_Trav+(sqrt(((LWing[n+1,0]-LWing[n,0])**2)+((LWing[n+1,1]-LWing[n,1])**2)+((LWing[n+1,2]-LWing[n,2])**2)))
                   

#    Returns values for left or right 
        
        StrokePlaneAngle=(degrees(StrokePlaneAngle))
        PositionGround=(degrees(Position_Ground))[R:S]
        ElevationGround=(degrees(WingAngle_Ground))[0,R:S]
        PositionStrokePlane=(degrees(Position))[R:S]
        ElevationStrokePlane=(degrees(WingAngle))[0,R:S]
        WingtipDist=(Wingtip_Dist_Trav)
        return StrokePlaneAngle,PositionGround,ElevationGround,PositionStrokePlane,ElevationStrokePlane,WingtipDist

##################################################################################################    


    
#Call programs
    #Create Summary variables

    Number=['']
    
    Body_Angle_DS=['Body_Angle']
    Body_Angle_YZ_DS=['Body_Angle_YZ']

    SA_Right_DS=['SA_Right_DS']
    SA_Left_DS=['SA_Left_DS']

    R_Amp_SP_DS=['R_Amp_SP_DS']
    L_Amp_SP_DS=['L_Amp_SP_DS']

    R_Wing_Elevation_Amp_DS=['R_Wing_Angle_Avg_DS']
    L_Wing_Elevation_Amp_DS=['L_Wing_Angle_Avg_DS']

    R_Wing_Angle_Avg_Ground_DS=['R_Wing_Angle_Avg_Ground_DS']
    L_Wing_Angle_Avg_Ground_DS=['L_Wing_Angle_Avg_Ground_DS']

    R_Wingtip_Dist_Trav_DS=['R_Wingtip_Dist_Trav_DS']
    L_Wingtip_Dist_Trav_DS=['L_Wingtip_Dist_Trav_DS']

    Roll_Angle_DS=['Roll_Angle_DS']

    
    Body_Angle_US=['Body_Angle']
    Body_Angle_YZ_US=['Body_Angle_YZ']

    SA_Right_US=['SA_Right_US']
    SA_Left_US=['SA_Left_US']

    R_Amp_SP_US=['R_Amp_SP_US']
    L_Amp_SP_US=['L_Amp_SP_US']

    R_Wing_Elevation_Amp_US=['R_Wing_Angle_Avg_US']
    L_Wing_Elevation_Amp_US=['L_Wing_Angle_Avg_US']

    R_Wing_Angle_Avg_Ground_US=['R_Wing_Angle_Avg_Ground_US']
    L_Wing_Angle_Avg_Ground_US=['L_Wing_Angle_Avg_Ground_US']

    R_Wingtip_Dist_Trav_US=['R_Wingtip_Dist_Trav_US']
    L_Wingtip_Dist_Trav_US=['L_Wingtip_Dist_Trav_US']
    
    Roll_Angle_US=['Roll_Angle_US']
    

#####################################################################################################################
    RAngleAttack_seq,LAngleAttack_seq=angleofattack(Transformed)

    Output[:,14]=RAngleAttack_seq
    Output[:,15]=LAngleAttack_seq
    Output[:,16]=Roll_Angle    

# Calculations per upstroke/downstroke
                
    for n in range(len(U)-1):        

    
# right wing        
        if n%2==0:
            Q = 'Down_'+str(n+1)+'_'+filename[:-4]+'_Right'
        else:
            Q = 'Up_'+str(n+1)+'_'+filename[:-4]+'_Right'

        StrokePlaneAngle,PositionGround,ElevationGround,PositionStrokePlane,ElevationStrokePlane,WingtipDist=StrokeAngle((U[n]-1),(U[n+1]-1),1)

        Output[(U[n]-1):(U[n+1]-1),2]=StrokePlaneAngle
        Output[(U[n]-1):(U[n+1]-1),3]=PositionGround
        Output[(U[n]-1):(U[n+1]-1),4]=ElevationGround
        Output[(U[n]-1):(U[n+1]-1),5]=PositionStrokePlane
        Output[(U[n]-1):(U[n+1]-1),6]=ElevationStrokePlane
        Output[(U[n]-1):(U[n+1]-1),7] = WingtipDist

# left wing
        if n%2==0:
            Q = 'Down_'+str(n+1)+'_'+filename[:-4]+'_Left'
        else:
            Q = 'Up_'+str(n+1)+'_'+filename[:-4]+'_Left'

        StrokePlaneAngle,PositionGround,ElevationGround,PositionStrokePlane,ElevationStrokePlane,WingtipDist=StrokeAngle((U[n]-1),(U[n+1]-1),2)

        Output[(U[n]-1):(U[n+1]-1),8]=StrokePlaneAngle   
        Output[(U[n]-1):(U[n+1]-1),9]=PositionGround    
        Output[(U[n]-1):(U[n+1]-1),10]=ElevationGround   
        Output[(U[n]-1):(U[n+1]-1),11]=PositionStrokePlane    
        Output[(U[n]-1):(U[n+1]-1),12]=ElevationStrokePlane   
        Output[(U[n]-1):(U[n+1]-1),13] = WingtipDist    
                      

        #Downstroke

        if n%2==0:
        
            Number.append(str((n/2)+1))
    
            Body_Angle_DS.append(average(Output[(U[n]-1):(U[n+1]-1),0]))
            Body_Angle_YZ_DS.append(average(Output[(U[n]-1):(U[n+1]-1),1]))

            SA_Right_DS.append(Output[U[n]-1,2])
            SA_Left_DS.append(Output[U[n]-1,8])
            
            R_Wing_Angle_Avg_Ground_DS.append(average(Output[(U[n]-1):(U[n+1]-1),4]))
            L_Wing_Angle_Avg_Ground_DS.append(average(Output[(U[n]-1):(U[n+1]-1),10]))

            R_Amp_SP_DS.append((max(Output[(U[n]-1):(U[n+1]-1),5]))-(min(Output[(U[n]-1):(U[n+1]-1),5])))
            L_Amp_SP_DS.append((max(Output[(U[n]-1):(U[n+1]-1),11]))-(min(Output[(U[n]-1):(U[n+1]-1),11])))

            
            R_Wing_Elevation_Amp_DS.append((max(Output[(U[n]-1):(U[n+1]-1),6]))-(min(Output[(U[n]-1):(U[n+1]-1),6])))
            L_Wing_Elevation_Amp_DS.append((max(Output[(U[n]-1):(U[n+1]-1),12]))-(min(Output[(U[n]-1):(U[n+1]-1),12])))

            R_Wingtip_Dist_Trav_DS.append(Output[U[n]-1,7])
            L_Wingtip_Dist_Trav_DS.append(Output[U[n]-1,13])

            Roll_Angle_DS.append(average(Roll_Angle[(U[n]-1):(U[n+1]-1)]))
            
        #Upstroke
    
        else:    

            Body_Angle_US.append(average(Output[(U[n]-1):(U[n+1]-1),0]))
            Body_Angle_YZ_US.append(average(Output[(U[n]-1):(U[n+1]-1),1]))

            SA_Right_US.append(Output[U[n]-1,2])
            SA_Left_US.append(Output[U[n]-1,8])
            
            R_Wing_Angle_Avg_Ground_US.append(average(Output[(U[n]-1):(U[n+1]-1),4]))
            L_Wing_Angle_Avg_Ground_US.append(average(Output[(U[n]-1):(U[n+1]-1),10]))
            
            R_Amp_SP_US.append((max(Output[(U[n]-1):(U[n+1]-1),5]))-(min(Output[(U[n]-1):(U[n+1]-1),5])))
            L_Amp_SP_US.append((max(Output[(U[n]-1):(U[n+1]-1),11]))-(min(Output[(U[n]-1):(U[n+1]-1),11])))

            R_Wing_Elevation_Amp_US.append((max(Output[(U[n]-1):(U[n+1]-1),6]))-(min(Output[(U[n]-1):(U[n+1]-1),6])))
            L_Wing_Elevation_Amp_US.append((max(Output[(U[n]-1):(U[n+1]-1),12]))-(min(Output[(U[n]-1):(U[n+1]-1),12])))

            R_Wingtip_Dist_Trav_US.append(Output[U[n]-1,7])
            L_Wingtip_Dist_Trav_US.append(Output[U[n]-1,13])

            Roll_Angle_US.append(average(Roll_Angle[(U[n]-1):(U[n+1]-1)]))

############################################################################################################################    

    Writer = csv.writer(open(path+"/Angles-sequence_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
    Writer.writerow(['Body_Angle_lateral','Body_Angle_dorsal','R_Stroke_Plane_Angle', 'R_Position_Angle_Ground','R_Elevation_Angle_Ground','R_Position_Angle_Stroke_Plane','R_Elevation_Angle_Stroke_Plane','R_Wingtip_Distance_Traveled(mm)','L_Stroke_Plane_Angle', 'L_Position_Angle_Ground','L_Elevation_Angle_Ground','L_Position_Angle_Stroke_Plane','L_Elevation_Angle_Stroke_Plane','L_Wingtip_Distance_Traveled(mm)','R_attack','L_attack','Roll_Angle'])
    for N in range(0,U[len(U)-1]-1):
        Writer.writerow(Output[N])

# store output in array

    # Change to convert wingtip velocity from mm/frame to whatever you want (10 = [1000fps*10 upsampled frames]/[1000mm/m])
    FrameMult=10
    #############

    Output2 = zeros((len(Body_Angle_DS)-1,36))

    Output2[:,0] = Number[1:]
    Output2[:,1] = Time_DS[1:]
    Output2[:,2] = Rotation_Angle_DS[1:]
    Output2[:,3] = Body_Angle_DS[1:]
    Output2[:,4] = Body_Angle_YZ_DS[1:]
    Output2[:,5] = Roll_Angle_DS[1:]
    Output2[:,6] = SA_Right_DS[1:]
    Output2[:,7] = SA_Left_DS[1:]
    Output2[:,8] = R_Wing_Angle_Avg_Ground_DS[1:]
    Output2[:,9] = L_Wing_Angle_Avg_Ground_DS[1:]
    Output2[:,10] = R_Amp_SP_DS[1:]
    Output2[:,11] = L_Amp_SP_DS[1:]
    Output2[:,12] = R_Wing_Elevation_Amp_DS[1:]
    Output2[:,13] = L_Wing_Elevation_Amp_DS[1:]


    Output2[:,14] = R_Wingtip_Dist_Trav_DS[1:]
    Output2[:,15] = (FrameMult*asarray(R_Wingtip_Dist_Trav_DS[1:])/asarray(Time_DS[1:]))
    Output2[:,16] = L_Wingtip_Dist_Trav_DS[1:]
    Output2[:,17] = (FrameMult*asarray(L_Wingtip_Dist_Trav_DS[1:])/asarray(Time_DS[1:]))


    Output2[:,18] = Number[1:]
    Output2[:,19] = Time_US[1:]
    Output2[:,20] = Rotation_Angle_US[1:]
    Output2[:,21] = Body_Angle_US[1:]
    Output2[:,22] = Body_Angle_YZ_US[1:]
    Output2[:,23] = Roll_Angle_US[1:]
    Output2[:,24] = SA_Right_US[1:]
    Output2[:,25] = SA_Left_US[1:]
    Output2[:,26] = R_Wing_Angle_Avg_Ground_US[1:]
    Output2[:,27] = L_Wing_Angle_Avg_Ground_US[1:]
    Output2[:,28] = R_Amp_SP_US[1:]
    Output2[:,29] = L_Amp_SP_US[1:]
    Output2[:,30] = R_Wing_Elevation_Amp_US[1:]
    Output2[:,31] = L_Wing_Elevation_Amp_US[1:]
    Output2[:,32] = R_Wingtip_Dist_Trav_US[1:]
    Output2[:,33] = (FrameMult*asarray(R_Wingtip_Dist_Trav_US[1:])/asarray(Time_US[1:]))
    Output2[:,34] = L_Wingtip_Dist_Trav_US[1:]
    Output2[:,35] = (FrameMult*asarray(L_Wingtip_Dist_Trav_US[1:])/asarray(Time_US[1:]))

#write to file


    Writer = csv.writer(open(path+"/Avg-angles_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
    Writer.writerow(["Wingbeat#","Wingbeat_Length_DS", "Travel_Angle_DS", "Body_Angle_Lateral_DS","Body_Angle_Dorsal_DS","Roll_Angle_DS","R_Stroke_Plane_Angle_DS","L_Stroke_Plane_Angle_DS","R_Avg_Elevation_Angle_Ground_DS","L_Avg_Elevation_Angle_Ground_DS", "R_Amplitude_Stroke_Plane_DS","L_Amplitude_Stroke_Plane_DS","R_Wing_Elevation_Amplitude_Stroke_Plane_DS","L_Wing_Elevation_Amplitude_Stroke_Plane_DS","R_Wingtip_Dist_Traveled_DS(mm)","R_Wingtip_Velocity_DS","L_Wingtip_Dist_Traveled_DS(mm)","L_Wingtip_Velocity_DS","Wingbeat#","Wingbeat_Length_US", "Travel_Angle_US", "Body_Angle_Lateral_US","Body_Angle_Dorsal_US","Roll_Angle_US","R_Stroke_Plane_Angle_US","L_Stroke_Plane_Angle_US","R_Avg_Elevation_Angle_Ground_US","L_Avg_Elevation_Angle_Ground_US", "R_Amplitude_Stroke_Plane_US","L_Amplitude_Stroke_Plane_US","R_Wing_Elevation_Amplitude_Stroke_Plane_US","L_Wing_Elevation_Amplitude_Stroke_Plane_US","R_Wingtip_Dist_Traveled_US","R_Wingtip_Velocity_US","L_Wingtip_Dist_Traveled_US","L_Wingtip_Velocity_US"])

    for N in range(len(Body_Angle_DS)-1):
        Writer.writerow(Output2[N])



###################################################################################################        

# Wingbeat Average Calculations

    def WingbeatAverage(O,P,Q,R):
    
        b=Transformed[(O):(P),(Q)]
        a=range(1,len(b)+1)        

        c= interpolate.splrep(a,b)
        d= linspace(1,(len(b)+1),num=100)
              
        R[:,(n/2)] = interpolate.splev(d,c)


# Average Wingbeat

    AvgWingbeat_DS=zeros((100,24,((len(U)-1)/2)))
    AvgWingbeat_US=zeros((100,24,((len(U)-1)/2)))

    for m in range(0,24): 
        for n in range((len(U)-1)/2):
            n=n*2
            WingbeatAverage((U[n]-1),(U[n+1]-1),m,AvgWingbeat_DS[:,m,:])
            WingbeatAverage((U[n+1]-1),(U[n+2]-1),m,AvgWingbeat_US[:,m,:])

    AvgWingbeatAvg_DS=average(AvgWingbeat_DS,axis=2)
    AvgWingbeatAvg_US=average(AvgWingbeat_US,axis=2)

    AvgWingbeat=hstack((AvgWingbeatAvg_DS,AvgWingbeatAvg_US))


    Writer = csv.writer(open(path+"/average_graphs/WingbeatAverage_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
    Writer.writerow(['RShoulder_DS_X','RShoulder_DS_Y','RShoulder_DS_Z','RPrimary_DS_X','RPrimary_DS_Y','RPrimary_DS_Z','RWing_DS_X','RWing_DS_Y','RWing_DS_Z','LShoulder_DS_X','LShoulder_DS_Y','LShoulder_DS_Z','LPrimary_DS_X','LPrimary_DS_Y','LPrimary_DS_Z','LWing_DS_X', 'LWing_DS_Y','LWing_DS_Z','Head_DS_X','Head_DS_Y','Head_DS_Z','Tail_DS_X','Tail_DS_Y','Tail_DS_Z','RShoulder_US_X','RShoulder_US_Y','RShoulder_US_Z','RPrimary_US_X','RPrimary_US_Y','RPrimary_US_Z','RWing_US_X','RWing_US_Y','RWing_US_Z','LShoulder_US_X','LShoulder_US_Y','LShoulder_US_Z','LPrimary_US_X','LPrimary_US_Y','LPrimary_US_Z','LWing_US_X', 'LWing_US_Y','LWing_US_Z','Head_US_X','Head_US_Y','Head_US_Z','Tail_US_X','Tail_US_Y','Tail_US_Z'])
    for N in range(0,100):
        Writer.writerow(AvgWingbeat[N])


# plot all points 


    clf()

    plot(AvgWingbeatAvg_DS[:,0],AvgWingbeatAvg_DS[:,2],c='g')
    plot(AvgWingbeatAvg_US[:,0],AvgWingbeatAvg_US[:,2],c='g')

    plot(AvgWingbeatAvg_DS[:,3],AvgWingbeatAvg_DS[:,5],c='c')
    plot(AvgWingbeatAvg_US[:,3],AvgWingbeatAvg_US[:,5],c='c')

    plot(AvgWingbeatAvg_DS[:,6],AvgWingbeatAvg_DS[:,8],c='b')
    plot(AvgWingbeatAvg_US[:,6],AvgWingbeatAvg_US[:,8],c='b')

    plot(AvgWingbeatAvg_DS[:,9],AvgWingbeatAvg_DS[:,11],c='y')
    plot(AvgWingbeatAvg_US[:,9],AvgWingbeatAvg_US[:,11],c='y')

    plot(AvgWingbeatAvg_DS[:,12],AvgWingbeatAvg_DS[:,14],c='m')
    plot(AvgWingbeatAvg_US[:,12],AvgWingbeatAvg_US[:,14],c='m')

    plot(AvgWingbeatAvg_DS[:,15],AvgWingbeatAvg_DS[:,17],c='r')
    plot(AvgWingbeatAvg_US[:,15],AvgWingbeatAvg_US[:,17],c='r')

    scatter(AvgWingbeatAvg_DS[:,18],AvgWingbeatAvg_DS[:,20],c='k')
    scatter(AvgWingbeatAvg_US[:,18],AvgWingbeatAvg_US[:,20],c='k')

    plot(AvgWingbeatAvg_DS[:,21],AvgWingbeatAvg_DS[:,23],c='Orange')
    plot(AvgWingbeatAvg_US[:,21],AvgWingbeatAvg_US[:,23],c='Orange')   

    axis((-85,65,-100,50))    
    axes().set_aspect('equal', 'datalim')
    grid(True)
    #show()    
    savefig(path+'/average_graphs/XZ_avg_'+filename[:-4]+'.svg')



    clf()
    plot(AvgWingbeatAvg_DS[:,1],AvgWingbeatAvg_DS[:,2],c='g')
    plot(AvgWingbeatAvg_US[:,1],AvgWingbeatAvg_US[:,2],c='g')

    plot(AvgWingbeatAvg_DS[:,4],AvgWingbeatAvg_DS[:,5],c='c')
    plot(AvgWingbeatAvg_US[:,4],AvgWingbeatAvg_US[:,5],c='c')

    plot(AvgWingbeatAvg_DS[:,7],AvgWingbeatAvg_DS[:,8],c='b')
    plot(AvgWingbeatAvg_US[:,7],AvgWingbeatAvg_US[:,8],c='b')

    plot(AvgWingbeatAvg_DS[:,10],AvgWingbeatAvg_DS[:,11],c='y')
    plot(AvgWingbeatAvg_US[:,10],AvgWingbeatAvg_US[:,11],c='y')

    plot(AvgWingbeatAvg_DS[:,13],AvgWingbeatAvg_DS[:,14],c='m')
    plot(AvgWingbeatAvg_US[:,13],AvgWingbeatAvg_US[:,14],c='m')

    plot(AvgWingbeatAvg_DS[:,16],AvgWingbeatAvg_DS[:,17],c='r')
    plot(AvgWingbeatAvg_US[:,16],AvgWingbeatAvg_US[:,17],c='r')

    scatter(AvgWingbeatAvg_DS[:,19],AvgWingbeatAvg_DS[:,20],c='k')
    scatter(AvgWingbeatAvg_US[:,19],AvgWingbeatAvg_US[:,20],c='k')

    plot(AvgWingbeatAvg_DS[:,22],AvgWingbeatAvg_DS[:,23],c='Orange')
    plot(AvgWingbeatAvg_US[:,22],AvgWingbeatAvg_US[:,23],c='Orange')  

    axis((-75,75,-100,50))    
    axes().set_aspect('equal', 'datalim')
    grid(True)
#    show()    
    savefig(path+'/average_graphs/YZ_avg_'+filename[:-4]+'.svg')





    clf()
    plot(AvgWingbeatAvg_DS[:,0],AvgWingbeatAvg_DS[:,1],c='g')
    plot(AvgWingbeatAvg_US[:,0],AvgWingbeatAvg_US[:,1],c='g')

    plot(AvgWingbeatAvg_DS[:,3],AvgWingbeatAvg_DS[:,4],c='c')
    plot(AvgWingbeatAvg_US[:,3],AvgWingbeatAvg_US[:,4],c='c')

    plot(AvgWingbeatAvg_DS[:,6],AvgWingbeatAvg_DS[:,7],c='b')
    plot(AvgWingbeatAvg_US[:,6],AvgWingbeatAvg_US[:,7],c='b')

    plot(AvgWingbeatAvg_DS[:,9],AvgWingbeatAvg_DS[:,10],c='y')
    plot(AvgWingbeatAvg_US[:,9],AvgWingbeatAvg_US[:,10],c='y')

    plot(AvgWingbeatAvg_DS[:,12],AvgWingbeatAvg_DS[:,13],c='m')
    plot(AvgWingbeatAvg_US[:,12],AvgWingbeatAvg_US[:,13],c='m')

    plot(AvgWingbeatAvg_DS[:,15],AvgWingbeatAvg_DS[:,16],c='r')
    plot(AvgWingbeatAvg_US[:,15],AvgWingbeatAvg_US[:,16],c='r')

    scatter(AvgWingbeatAvg_DS[:,18],AvgWingbeatAvg_DS[:,19],c='k')
    scatter(AvgWingbeatAvg_US[:,18],AvgWingbeatAvg_US[:,19],c='k')

    plot(AvgWingbeatAvg_DS[:,21],AvgWingbeatAvg_DS[:,22],c='Orange')
    plot(AvgWingbeatAvg_US[:,21],AvgWingbeatAvg_US[:,22],c='Orange')  

    axis((-85,65,-75,75))
    axes().set_aspect('equal', 'datalim')
    grid(True)
#    show()    
    savefig(path+'/average_graphs/XY_avg_'+filename[:-4]+'.svg')
####################################################################################################
  

# Per Wingbeat Calculations
    clf()
    def PerWingbeat(O,P,Q,R):

        b=Output[(O):(P),(Q)]
        a=range(1,len(b)+1)        

        c= interpolate.splrep(a,b)
        d= linspace(1,(len(b)+1),num=100)
        
        
        R[:,0] = linspace(1,25,num=100)        
        R[:,((n/2)+1)] = interpolate.splev(d,c)

######################################################################################################################

# Position and Deviation from Horizontal

    RPosition_GR_DS= zeros((100,((len(U)-1)/2)+1))        
    RDeviation_GR_DS= zeros((100,((len(U)-1)/2)+1))
    LPosition_GR_DS= zeros((100,((len(U)-1)/2)+1))        
    LDeviation_GR_DS= zeros((100,((len(U)-1)/2)+1))
    RAttack_DS= zeros((100,((len(U)-1)/2)+1))        
    LAttack_DS= zeros((100,((len(U)-1)/2)+1))    
    RAngVel_DS= zeros((100,((len(U)-1)/2)+1))        
    LAngVel_DS= zeros((100,((len(U)-1)/2)+1))
    RAttVel_DS= zeros((100,((len(U)-1)/2)+1))        
    LAttVel_DS= zeros((100,((len(U)-1)/2)+1))

    for n in range((len(U)-1)/2):
        n=n*2
        PerWingbeat((U[n]-1),(U[n+1]-1),3,RPosition_GR_DS)
        PerWingbeat((U[n]-1),(U[n+1]-1),4,RDeviation_GR_DS)
        PerWingbeat((U[n]-1),(U[n+1]-1),9,LPosition_GR_DS)
        PerWingbeat((U[n]-1),(U[n+1]-1),10,LDeviation_GR_DS)
        PerWingbeat((U[n]-1),(U[n+1]-1),14,RAttack_DS)
        PerWingbeat((U[n]-1),(U[n+1]-1),15,LAttack_DS)

        multiplier=10*100.0/((U[n+1]-1)-(U[n]-1))  #10*upsampled steps/frames; use this to convert angular velocities to units of degrees/ms

        RAngVel_DS[1:,((n/2)+1)]=abs(diff(degrees(arccos(cos(radians(RPosition_GR_DS[:,((n/2)+1)]))*cos(radians(RDeviation_GR_DS[:,((n/2)+1)]))))))*multiplier
        LAngVel_DS[1:,((n/2)+1)]=abs(diff(degrees(arccos(cos(radians(RPosition_GR_DS[:,((n/2)+1)]))*cos(radians(LDeviation_GR_DS[:,((n/2)+1)]))))))*multiplier

        RAttVel_DS[1:,((n/2)+1)]=diff(RAttack_DS[:,((n/2)+1)])*multiplier
        LAttVel_DS[1:,((n/2)+1)]=diff(LAttack_DS[:,((n/2)+1)])*multiplier

        #print multiplier 

#    Calculate Averages and save as column 1

    for n in range(0,100):
        
        RPosition_GR_DS[n,0]=average(RPosition_GR_DS[n,1:])
        RDeviation_GR_DS[n,0]=average(RDeviation_GR_DS[n,1:])
        RAttack_DS[n,0]=average(RAttack_DS[n,1:])
        RAngVel_DS[n,0]=average(RAngVel_DS[n,1:])
        RAttVel_DS[n,0]=average(RAttVel_DS[n,1:]) 

        LPosition_GR_DS[n,0]=average(LPosition_GR_DS[n,1:])
        LDeviation_GR_DS[n,0]=average(LDeviation_GR_DS[n,1:])
        LAttack_DS[n,0]=average(LAttack_DS[n,1:])
        LAngVel_DS[n,0]=average(LAngVel_DS[n,1:])
        LAttVel_DS[n,0]=average(LAttVel_DS[n,1:]) 

##################upstroke

    RPosition_GR_US= zeros((100,((len(U)-1)/2)+1))        
    RDeviation_GR_US= zeros((100,((len(U)-1)/2)+1))
    LPosition_GR_US= zeros((100,((len(U)-1)/2)+1))        
    LDeviation_GR_US= zeros((100,((len(U)-1)/2)+1))
    RAttack_US= zeros((100,((len(U)-1)/2)+1))        
    LAttack_US= zeros((100,((len(U)-1)/2)+1))
    RAngVel_US= zeros((100,((len(U)-1)/2)+1))        
    LAngVel_US= zeros((100,((len(U)-1)/2)+1))
    RAttVel_US= zeros((100,((len(U)-1)/2)+1))        
    LAttVel_US= zeros((100,((len(U)-1)/2)+1))
    
    for n in range((len(U)-1)/2):
        n=n*2
        PerWingbeat((U[n+1]-1),(U[n+2]-1),3,RPosition_GR_US)
        PerWingbeat((U[n+1]-1),(U[n+2]-1),4,RDeviation_GR_US)
        PerWingbeat((U[n+1]-1),(U[n+2]-1),9,LPosition_GR_US)
        PerWingbeat((U[n+1]-1),(U[n+2]-1),10,LDeviation_GR_US)
        PerWingbeat((U[n+1]-1),(U[n+2]-1),14,RAttack_US)
        PerWingbeat((U[n+1]-1),(U[n+2]-1),15,LAttack_US)

        multiplier = 10*100.0/((U[n+2]-1)-(U[n+1]-1))  #10*upsampled steps/frames; use this to convert angular velocities to units of degrees/ms

        RAngVel_US[1:,((n/2)+1)]=abs(diff(degrees(arccos(cos(radians(RPosition_GR_US[:,((n/2)+1)]))*cos(radians(RDeviation_GR_US[:,((n/2)+1)]))))))*multiplier
        LAngVel_US[1:,((n/2)+1)]=abs(diff(degrees(arccos(cos(radians(LPosition_GR_US[:,((n/2)+1)]))*cos(radians(LDeviation_GR_US[:,((n/2)+1)]))))))*multiplier
 
        RAttVel_US[1:,((n/2)+1)]=diff(RAttack_US[:,((n/2)+1)])*multiplier
        LAttVel_US[1:,((n/2)+1)]=diff(LAttack_US[:,((n/2)+1)])*multiplier

#    Calculate Averages and save as column 1

    for n in range(0,100):
        
        RPosition_GR_US[n,0]=average(RPosition_GR_US[n,1:])
        RDeviation_GR_US[n,0]=average(RDeviation_GR_US[n,1:])
        RAttack_US[n,0]=average(RAttack_US[n,1:])
        RAngVel_US[n,0]=average(RAngVel_US[n,1:])
        RAttVel_US[n,0]=average(RAttVel_US[n,1:]) 
  

        LPosition_GR_US[n,0]=average(LPosition_GR_US[n,1:])
        LDeviation_GR_US[n,0]=average(LDeviation_GR_US[n,1:])
        LAttack_US[n,0]=average(LAttack_US[n,1:])
        LAngVel_US[n,0]=average(LAngVel_US[n,1:])
        LAttVel_US[n,0]=average(LAttVel_US[n,1:])  

# Consolidate

    RPosition_GR = vstack((RPosition_GR_DS,RPosition_GR_US[1:,:]))
    LPosition_GR = vstack((LPosition_GR_DS,LPosition_GR_US[1:,:]))
    RDeviation_GR = vstack((RDeviation_GR_DS,RDeviation_GR_US[1:,:]))
    LDeviation_GR = vstack((LDeviation_GR_DS,LDeviation_GR_US[1:,:]))
    RAttack = vstack((RAttack_DS,RAttack_US[1:,:]))
    LAttack = vstack((LAttack_DS,LAttack_US[1:,:]))
    RAngVel = vstack((RAngVel_DS,RAngVel_US[1:,:]))
    LAngVel = vstack((LAngVel_DS,LAngVel_US[1:,:]))
    RAttVel = vstack((RAttVel_DS,RAttVel_US[1:,:]))
    LAttVel = vstack((LAttVel_DS,LAttVel_US[1:,:]))

#  Instantaneous angles

    Writer = csv.writer(open(path+"/Instant-angles_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)  
    Writer.writerow(("wingbeat","R_Position_DS_mid","R_Position_DS_end","R_Position_US_mid","R_Position_US_end",\
                                "R_Elevation_DS_mid","R_Elevation_DS_end","R_Elevation_US_mid","R_Elevation_US_end",\
                                "R_AngleAttack_DS_mid","R_AngleAttack_DS_end","R_AngleAttack_US_mid","R_AngleAttack_US_end",\
                                "R_AngVel_DS_mid","R_AngVel_DS_end","R_AngVel_US_mid","R_AngVel_US_end",\
                                "R_AttVel_DS_mid","R_AttVel_DS_end","R_AttVel_US_mid","R_AttVel_US_end",\
                                "L_Position_DS_mid","L_Position_DS_end","L_Position_US_mid","L_Position_US_end",\
                                "L_Elevation_DS_mid","L_Elevation_DS_end","L_Elevation_US_mid","L_Elevation_US_end",\
                                "L_AngleAttack_DS_mid","L_AngleAttack_DS_end","L_AngleAttack_US_mid","L_AngleAttack_US_end",\
                                "L_AngVel_DS_mid","L_AngVel_DS_end","L_AngVel_US_mid","L_AngVel_US_end",\
                                "L_AttVel_DS_mid","L_AttVel_DS_end","L_AttVel_US_mid","L_AttVel_US_end"))

    for n in range(1,len(RAttack[0,:])):
       instant= hstack((n,RPosition_GR[49,n],RPosition_GR[99,n],RPosition_GR[148,n],RPosition_GR[198,n],\
                RDeviation_GR[49,n],RDeviation_GR[99,n],RDeviation_GR[148,n],RDeviation_GR[198,n],\
                RAttack[49,n],RAttack[99,n],RAttack[148,n],RAttack[198,n],\
                RAngVel[49,n],RAngVel[99,n],RAngVel[148,n],RAngVel[198,n],\
                RAttVel[49,n],RAttVel[99,n],RAttVel[148,n],RAttVel[198,n],\
                LPosition_GR[49,n],LPosition_GR[99,n],LPosition_GR[148,n],LPosition_GR[198,n],\
                LDeviation_GR[49,n],LDeviation_GR[99,n],LDeviation_GR[148,n],LDeviation_GR[198,n],\
                LAttack[49,n],LAttack[99,n],LAttack[148,n],LAttack[198,n],\
                LAngVel[49,n],LAngVel[99,n],LAngVel[148,n],LAngVel[198,n],\
                LAttVel[49,n],LAttVel[99,n],LAttVel[148,n],LAttVel[198,n]))
       Writer.writerow((instant))

#     graph        

    clf()
    plot(RDeviation_GR[:,1],c='b')
    plot(LDeviation_GR[:,1],c='r')
    axis((0,200,-45,45))
    savefig(path+'/wingbeat_timecourse/LRElevation_GR_'+filename[:-4])
#    show()

    clf()
    plot(RPosition_GR[:,1],c='b')
    plot(LPosition_GR[:,1],c='r')
    axis((0,200,-10,190))
    savefig(path+'/wingbeat_timecourse/LRPosition_GR_'+filename[:-4])
#    show()

    clf()
    plot(RAttack[:,1],c='b')
    plot(LAttack[:,1],c='r')
    axis((0,200,0,180))

    savefig(path+'/wingbeat_timecourse/LRAttack_'+filename[:-4])

    clf()
    plot(RAngVel[1:,1],c='b')
    plot(LAngVel[1:,1],c='r')
    axis((0,200,0,45))
    savefig(path+'/wingbeat_timecourse/AngVel_'+filename[:-4])

    clf()
    plot(RAttVel[1:,1],c='b')
    plot(LAttVel[1:,1],c='r')
    axis((0,200,-45,45))
    savefig(path+'/wingbeat_timecourse/AttVel_'+filename[:-4])



#    Write Output
    if 0:
        header = ['Average']
        for n in range(3,len(RDeviation_GR[1,:])):
            header.append(n-2)

        Writer = csv.writer(open(path+"/wingbeat_timecourse/LAngleVel_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
        Writer.writerow(header)
        for N in range(0,199):
            Writer.writerow(LAngVel[N])

        Writer = csv.writer(open(path+"/wingbeat_timecourse/RAttVel_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
        Writer.writerow(header)
        for N in range(0,199):
            Writer.writerow(RAttVel[N])

        Writer = csv.writer(open(path+"/wingbeat_timecourse/RPosition_GR_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
        Writer.writerow(header)
        for N in range(0,199):
            Writer.writerow(RPosition_GR[N])

        Writer = csv.writer(open(path+"/wingbeat_timecourse/RElevation_GR_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
        Writer.writerow(header)
        for N in range(0,199):
            Writer.writerow(RDeviation_GR[N])

  
################################

    LRPosElevAtt_Summary=hstack((RPosition_GR[:,0:1],RDeviation_GR[:,0:1],RAttack[:,0:1],RAngVel[:,0:1],RAttVel[:,0:1],LPosition_GR[:,0:1],LDeviation_GR[:,0:1],LAttack[:,0:1],LAngVel[:,0:1],LAttVel[:,0:1]))    

    Writer = csv.writer(open(path+"/wingbeat_timecourse/Summary_LRPosElevAtt_GR_"+filename, 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
    Writer.writerow(["R_Pos","R_Dev","R_Attack","R_Ang_vel","R_Att_vel","L_Pos","L_Dev","L_Attack","L_Ang_vel","L_Att_vel"])
    for N in range(0,199):
        Writer.writerow(LRPosElevAtt_Summary[N])

######################################################################################################################################################

if __name__ == '__main__':

        try:
            opts, args = getopt.getopt(sys.argv[1:], 'h', ['help'])


        except getopt.error:
            sys.stderr.write("%s: %s \nTry `%s --help` for more informatIon\n"
            % (PROGNAME, e, PROGNAME))
            sys.exit(1)


        for o in opts:
            if o in ('-h', '--help'):
                usage(error=0)

        if len(args) != 2:
            usage()

        main(args[0],args[1])
