#!/usr/bin/env python
from numpy import *
from math import *
from scipy import interpolate
import scipy
import os
import getopt
import sys
import csv
from pylab import *


def usage(error=1):
    usage = """\

Usage: %s [INPUT_FILE_NAME][INPUT_FILE_NAME2]

Requires the same inputs as 3_wingkinematics_arc_v7.0.py. Generates a file called "com/INPUT_FILE_NAME_com.csv" which contains XYZ coordinates of a reference point in a global frame of reference - in this case the head.

Options:
   -h, --help    Print this message

[written by Paolo Segre, 2015]


""" % os.path.basename(sys.argv[0])

    if error:
        sys.stderr.write(usage)
        sys.exit(1)
    else:
        sys.stdout.write(usage)
        sys.exit()



def main(file,time):
    seterr(all='ignore')
    (path, filename) = os.path.split(file)

# input files

    inFile = open(file,"r")
    lines = inFile.readlines()    

    inFile2 = open(time,"r")
    lines2 = inFile2.readlines()    


    Data= zeros(((len(lines)-1),30), float)



    for N in range(1,len(lines)):

        temp = str(lines[N]).split(',')            

        Data[N-1,:]=(float(temp[0]),float(temp[1]),float(temp[2]),float(temp[3]),float(temp[4]),float(temp[5]),float(temp[6]),float(temp[7]),float(temp[8]),float(temp[9]),float(temp[10]),float(temp[11]),float(temp[12]),float(temp[13]),float(temp[14]),float(temp[15]),float(temp[16]),float(temp[17]),float(temp[18]),float(temp[19]),float(temp[20]),float(temp[21]),float(temp[22]),float(temp[23]),float(temp[24]),float(temp[25]),float(temp[26]),float(temp[27]),float(temp[28]),float(temp[29]))
    


    temp2= str(lines2[1]).split(',')
    U=[]
    
    for N in range(0,len(temp2)):    
        U.append(int(temp2[N]))        
        

# Make directory for figures
    os.system("mkdir %s"%(path+"/com"))

# Define Output Arrays

    Output= zeros(((U[len(U)-1]-1),17))

# Read data from input array


    RSHOULDER= zeros(((len(lines)-1),3))
    RPRIMARY= zeros(((len(lines)-1),3))
    RWING= zeros(((len(lines)-1),3))
    LSHOULDER= zeros(((len(lines)-1),3))
    LPRIMARY= zeros(((len(lines)-1),3))
    LWING= zeros(((len(lines)-1),3))
    HEAD= zeros(((len(lines)-1),3))
    TAIL= zeros(((len(lines)-1),3))
    RBACK= zeros(((len(lines)-1),3))
    LBACK= zeros(((len(lines)-1),3))
    

    RSHOULDER[:,0:3]=Data[:,0:3]
    RPRIMARY[:,0:3]=Data[:,3:6]
    RWING[:,0:3]=Data[:,6:9]
    LSHOULDER[:,0:3]=Data[:,9:12]
    LPRIMARY[:,0:3]=Data[:,12:15]
    LWING[:,0:3]=Data[:,15:18]
    HEAD[:,0:3]=Data[:,18:21]
    TAIL[:,0:3]=Data[:,21:24]    
    RBACK[:,0:3]=Data[:,24:27]
    LBACK[:,0:3]=Data[:,27:30]


# Center head at 0,0,- = subtract head (XY) coordinates from all vectors 

    Com1=HEAD[1:,:]-HEAD[:-1,:]
    Com1=vstack((Com1,[0,0,0]))

    Head1=HEAD-HEAD
    Head1[:,2]=HEAD[:,2]
    
    RShoulder1=RSHOULDER-HEAD    
    RShoulder1[:,2]=RSHOULDER[:,2]
    RPrimary1=RPRIMARY-HEAD
    RPrimary1[:,2]=RPRIMARY[:,2]    
    RWing1=RWING-HEAD
    RWing1[:,2]=RWING[:,2]
    
    LShoulder1=LSHOULDER-HEAD
    LShoulder1[:,2]=LSHOULDER[:,2]
    LPrimary1=LPRIMARY-HEAD
    LPrimary1[:,2]=LPRIMARY[:,2]
    LWing1=LWING-HEAD
    LWing1[:,2]=LWING[:,2]
    Tail1=TAIL-HEAD
    Tail1[:,2]=TAIL[:,2]

    RBack1=RBACK-HEAD
    RBack1[:,2]=RBACK[:,2]

    LBack1=LBACK-HEAD
    LBack1[:,2]=LBACK[:,2]



# Define global output data matrices
    Com= zeros(((U[len(U)-1]-1),3))
    RShoulder= zeros(((U[len(U)-1]-1),3))
    RPrimary= zeros(((U[len(U)-1]-1),3))
    RWing= zeros(((U[len(U)-1]-1),3))
    LShoulder= zeros(((U[len(U)-1]-1),3))
    LPrimary= zeros(((U[len(U)-1]-1),3))
    LWing= zeros(((U[len(U)-1]-1),3))
    Head= zeros(((U[len(U)-1]-1),3))
    Tail= zeros(((U[len(U)-1]-1),3))
    RBack= zeros(((U[len(U)-1]-1),3))
    LBack= zeros(((U[len(U)-1]-1),3))



# Transform all coordinates to Wingtip frame


    Rotation_Angle = ['Rotation_Angle']
    Rotation_Angle_DS = ['Rotation_Angle_DS']
    Rotation_Angle_US =['Rotation_Angle_US']    
    Time_Wingbeat = ['Time_Wingbeat']
    Time_DS =['Time_DS']
    Time_US    =['Time_US']    


   # Calculate Rotation angle for every downstroke and upstroke

    for n in range((len(U)-1)):
        
        
        A = ((U[n]-1))    
        B = (U[n+1]-1)    
        Midpoint_Start = (RWing1[A]+LWing1[A])/2
        Midpoint_Start[2]=0

        Midpoint_End = (RWing1[B]+LWing1[B])/2
        Midpoint_End[2] = 0

        if n%2==0:
            Mid = (Midpoint_End-Midpoint_Start)
        else:
            Mid = (Midpoint_Start-Midpoint_End)
        
        LengthMid= sqrt(Mid[0]**2 + Mid[1]**2)

        Xaxis=(1,0,0)
        
    
        
   # Calculate rotation angle between Midpoint and Xaxis vector. Create adjustment factor for acos: if Y is positive, subtract angle from 360 degrees to find rotation angle        
        
        if Mid[1]<0:
            ZRotationAngle= radians(360)-acos(((dot(Mid,Xaxis))/(LengthMid)))
        
        else: 
            ZRotationAngle= acos(((dot(Mid,Xaxis))/(LengthMid)))


        Rotation_Angle.append(360-(degrees(ZRotationAngle)))
        Time_Wingbeat.append(B-A)
    

#    Create rotation matrix, and multiply times Body1 and RShoulder1 vectors

        RotZ = array(((cos(ZRotationAngle),sin(ZRotationAngle),0),(-sin(ZRotationAngle),cos(ZRotationAngle),0),(0,0,1)))
        
        Com2=dot(RotZ,transpose(Com1[A:B,:]))
        RShoulder2=dot(RotZ,transpose(RShoulder1[A:B,:]))
        RPrimary2=dot(RotZ,transpose(RPrimary1[A:B,:]))
        RWing2=dot(RotZ,transpose(RWing1[A:B,:]))
        LShoulder2=dot(RotZ,transpose(LShoulder1[A:B,:]))
        LPrimary2=dot(RotZ,transpose(LPrimary1[A:B,:]))
        LWing2=dot(RotZ,transpose(LWing1[A:B,:]))
        Head2=dot(RotZ,transpose(Head1[A:B,:]))
        Tail2=dot(RotZ,transpose(Tail1[A:B,:]))
        RBack2=dot(RotZ,transpose(RBack1[A:B,:]))
        LBack2=dot(RotZ,transpose(LBack1[A:B,:]))


        Com3=transpose(Com2)
        RShoulder3=transpose(RShoulder2)
        RPrimary3=transpose(RPrimary2)
        RWing3=transpose(RWing2)
        LWing3=transpose(LWing2)
        LPrimary3=transpose(LPrimary2)
        LShoulder3=transpose(LShoulder2)        
        Head3=transpose(Head2)
        Tail3=transpose(Tail2)
        RBack3=transpose(RBack2)
        LBack3=transpose(LBack2)

        Com[A:B,:]=Com3[:,:]
        RShoulder[A:B,:]=RShoulder3[:,:]
        RPrimary[A:B,:]=RPrimary3[:,:]
        RWing[A:B,:]=RWing3[:,:]
        LShoulder[A:B,:]=LShoulder3[:,:]
        LPrimary[A:B,:]=LPrimary3[:,:]
        LWing[A:B,:]= LWing3[:,:]
        Head[A:B,:]= Head3[:,:]
        Tail[A:B,:]=Tail3[:,:]
        RBack[A:B,:]=RBack3[:,:]           
        LBack[A:B,:]=LBack3[:,:]


# Create output array for transformed coordinates

    Com_cumulative=vstack((cumsum(Com[:,0]), cumsum(Com[:,1]), cumsum(Com[:,2]))).T

    Com_cumulative=vstack(([0,0,0],Com_cumulative[:-1,:]))

    

#TRANSFORMATION COMPLETE
    if 1:
        clf()
        scatter(Com_cumulative[U[0]:U[-1],0],Com_cumulative[U[0]:U[-1],1],c='r')
        plot(LPrimary[U[0]:U[-1],0],LPrimary[U[0]:U[-1],1],c='m')
        plot(LWing[U[0]:U[-1],0],LWing[U[0]:U[-1],1],c='r')
        plot(LShoulder[U[0]:U[-1],0],LShoulder[U[0]:U[-1],1],c='y')
        plot(RPrimary[U[0]:U[-1],0],RPrimary[U[0]:U[-1],1],c='c')
        plot(RWing[U[0]:U[-1],0],RWing[U[0]:U[-1],1],c='b')
        plot(RShoulder[U[0]:U[-1],0],RShoulder[U[0]:U[-1],1],c='g')
        plot(Head[U[0]:U[-1],0],Head[U[0]:U[-1],1],c='k')
        plot(Tail[U[0]:U[-1],0],Tail[U[0]:U[-1],1],c='Orange')
        plot(RBack[U[0]:U[-1],0],RBack[U[0]:U[-1],1],c='c')
        plot(LBack[U[0]:U[-1],0],LBack[U[0]:U[-1],1],c='m')
    
        axis((-85,65,-75,75))
        axes().set_aspect('equal', 'datalim')
        grid(True)
#       show()    
        savefig(path+'/com/XY_com_'+filename[:-4]+'.png')

#       Write COM Coordinates
        Writer = csv.writer(open(path+"/com/"+filename[:-4]+"_com.csv", 'w'), delimiter=',', quotechar=',', quoting=csv.QUOTE_MINIMAL)
        Writer.writerow(['Com_X','Com_Y','Com_Z'])
        for N in range(0,U[len(U)-1]-1):
            Writer.writerow(Com_cumulative[N])

######################################################################################################################################################

if __name__ == '__main__':

        try:
            opts, args = getopt.getopt(sys.argv[1:], 'h', ['help'])


        except getopt.error:
            sys.stderr.write("%s: %s \nTry `%s --help` for more informatIon\n"
            % (PROGNAME, e, PROGNAME))
            sys.exit(1)


        for o in opts:
            if o in ('-h', '--help'):
                usage(error=0)

        if len(args) != 2:
            usage()

        main(args[0],args[1])
